#!/usr/bin/env python

import os, sys, glob, math, copy, time

try:
	from optparse import OptionParser
except:
	print "You need Python version 2.3 or later. Check http://www.python.org for more information"
	sys.exit(-1)
try:
	from Numeric import *
except:
	print "You need to install Numeric Python. Check http://numpy.sf.net for more information"
	sys.exit(-1)
try:
	import EMAN
except:
	print "You need to install EMAN python wrapper. Check http://ncmi.bcm.tmc.edu/homes/stevel/EMAN/doc for more infomation"
	sys.exit(-1)

def main():
	(options,args) =  parse_command_line()
	
	mgraphs = findAllImages(args)
	if not mgraphs: 
		print "%s: no micrograph/ccd images specified or found in current directory" % (sys.argv[0])
		sys.exit(1)

	# write log info to .emanlog file so that eman program can browse the history
	pid = LOGbegin(sys.argv)
	for m in mgraphs: LOGInfile(pid,m[1])
	
	if options.fitsfile: LOGOutfile(pid,options.fitsfile)
	else: LOGOutfile(pid,options.ctffile)
	
	tnf = EMAN.EMData()	# to load tnf file and save parameters in the tnf file header
	
	if options.useMatlabEngine:
		try:
			import pymat
		except:
			print "You need to install pymat package. Check http://pymat.sf.net for more information"
			sys.exit(-1)
		# start a Matlab engine session on local machine
		H = pymat.open("matlab -nosplash -nodesktop -nojvm")
		if not H:
			print "Warning: cannot start Matlab engine, I will run Matlab directly"
	else:
		H = None

	# now start fitting the structural factors
	if options.fitsfile: 
		if not options.useoldmfile:
  			temp_matlab_script_file = "fitsfall.m"
			fp = open(temp_matlab_script_file,"w")
			fp.write(matlab_sf_fit)
			fp.close()

		powerSpectrum_files = []
		for i in range(len(mgraphs)):
			mid = mgraphs[i][0]
			if options.gridbox:
				fftavg = gridBoxAverageFFT(mgraphs[i][1], options.gridbox)
			else:
				fftavg = calcAverageFFT(mgraphs[i][1], oversample = options.over)
			power = calcPowerSpectrum(fftavg, bins = options.anisobins)
			tempfile = "%s.pwr" % (mid)
			EMAN.save_data(0,1.0/(options.apix*2.0*len(power[0])),power[0],len(power[0]),tempfile)
			powerSpectrum_files.append(mid)
		fitsf_by_Matlab(powerSpectrum_files, options, engineConnection = None)
		sys.exit(0)
	
	# now start processing each micrograph/ccd using existing structural factors
	if not options.sfile:
		options.sfile = "fitctf.const.sf.dat"
		sffp = open(options.sfile,'w')
		sffp.write("0\t1\n1\t1\n")
		sffp.close()

	if not options.useoldmfile:
		temp_matlab_script_file = "fitlogctfc.m"
		fp = open(temp_matlab_script_file,"w")
		fp.write(matlab_1d_ctf_fit)
		fp.close()

		if  options.anisobins:
			temp_matlab_script_file = "fit_sin.m"
			fp = open(temp_matlab_script_file,"w")
			fp.write(matlab_fit_sin)
			fp.close()

	ctffile = open(options.ctffile,'a')
	
	for i in range(len(mgraphs)):
		mid = mgraphs[i][0]
		print "Fitting %d/%d images: %s" % (i, len(mgraphs), mgraphs[i][1])
		if options.gridbox:
			fftavg = gridBoxAverageFFT(mgraphs[i][1], options.gridbox)
		else:
			fftavg = calcAverageFFT(mgraphs[i][1], oversample = options.over)
		power = calcPowerSpectrum(fftavg, bins = options.anisobins)
		tempfile = "%s.pwr" % (mid)
		EMAN.save_data(0,1.0/(options.apix*2.0*len(power[0])),power[0],len(power[0]),tempfile)
		
		par = fitctf_by_Matlab(powerSpectrum_file=tempfile, options=options, lb = None, ub = None, par0=None, numTries = 5, engineConnection = H)		
		defocus_mean = par[0]
		B_mean = par[1]

		if options.Bexp:
			ctfparm =	'%s\t%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%s\n' % \
            		(mid,par[0],-par[1],par[2],math.sin(par[3]),par[4],par[5],par[6],par[7],options.volt,options.cs,options.apix,options.sfile)
		else:
			ctfparm =	'%s\t%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%s\n' % \
            		(mid,par[0],par[1],par[2],math.sin(par[3]),par[4],par[5],par[6],par[7],options.volt,options.cs,options.apix,options.sfile)
		print "\t%s" % ( ctfparm )
		
		# save the parameters to the tnf file header
		tnffile = "%s.tnf" % ( os.path.splitext(mgraphs[i][1])[0] )
		tnf.readImage(tnffile)
		if options.Bexp:
			tnf.setCTF([par[0],-par[1],par[2],math.sin(par[3]),par[4],par[5],par[6],par[7],options.volt,options.cs,options.apix])
		else:
			tnf.setCTF([par[0],par[1],par[2],math.sin(par[3]),par[4],par[5],par[6],par[7],options.volt,options.cs,options.apix])
		tnf.setSFFile(options.sfile)
		tnf.writeImage(tnffile, -1)
		
		if not options.keep: os.remove(tempfile)

		# now start fitting the astigmatism
		if not ( options.astig or options.drift):
			ctffile.write( ctfparm )
		else:
			aniso_x=[]; astig_y=[];	drift_y=[]
			lb   = '[  %f   0     0    %f -Inf -Inf -Inf -Inf]' % (par[0]*1.5, options.min_ac)
			ub   = '[  %f Inf   Inf    %f  Inf  Inf  Inf  Inf]' % (par[0]*0.5, options.max_ac)
			par0 = '[ %f %f %f %f %f %f %f %f ]' % ( par[0],par[1],par[2],par[3],par[4],par[5],par[6],par[7] )

			for bin in range(options.anisobins):
				print "\tFitting %d/%d angular bins: %s" % (bin, options.anisobins, mgraphs[i][1])
				mid2 = "%s-%d-%d" % (mid,bin,options.anisobins)
				tempfile = "%s.pwr" % (mid2)
				EMAN.save_data(0,1.0/(options.apix*2.0*len(power[bin+1])),power[bin+1],len(power[bin+1]),tempfile)

				par_bin = fitctf_by_Matlab(powerSpectrum_file=tempfile, options=options, lb = lb, ub = ub, par0=par0, numTries = 1, engineConnection = H)		
				aniso_x.append(bin*math.pi*2/options.anisobins)	# angle
				astig_y.append(par_bin[0])	# defocus
				drift_y.append(par_bin[1])	# B factor
				
				ctfparm_bin =	'%s\t%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%s\n' % \
            				(mid2,par_bin[0],par_bin[1],par_bin[2],math.sin(par_bin[3]),par_bin[4],par_bin[5],par_bin[6],par_bin[7],options.volt,options.cs,options.apix,options.sfile)
				print "\t\t%s" % ( ctfparm_bin )
			
				if not options.keep: os.remove(tempfile)
			
			if options.astig:
			
				if options.show_fit: figureID=2
				else: figureID=0
				
				# now fit defocus angular distribution to a sin function: a+b*sin(2(theta-theta0))
				astpar = fitAstigDrift_by_Matlab(aniso_x, astig_y, defocus_mean, engineConnection = H)
				
				#print "aniso_x", aniso_x
				#print "astig_y", astig_y
				#print "fitted sin:", astpar

				defocus_mean = astpar[0]	# use the mean defocus and convert to the same sign convention: negative for under defocus
				defocus_amp = astpar[1]		# use the mean defocus and convert to the same sign convention: negative for under defocus
				defocus_ratio = defocus_amp/fabs(defocus_mean)
				defocus_theta = astpar[2]+math.pi/4	# covert to convention: counter-clockwise from +X is positive. EMAN ctfit use the smaller defocus direction as the major axis direction
				
				print "\tAstigmatism: mean defocus = %g\tamp = %g\tratio = %g\tsmaller defocus axis angle = %g" % (defocus_mean, defocus_amp, defocus_ratio, defocus_theta*180./math.pi)

				ctfparm =	'%s\t%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%s\n' % \
            					(mid,defocus_mean+defocus_amp,2*defocus_amp,defocus_theta*180./math.pi,par[1],par[2],math.sin(par[3]),par[4],par[5],par[6],par[7],options.volt,options.cs,options.apix,options.sfile)
            					#(mid,par[0],defocus_e,ellipse_theta,par[1],par[2],math.sin(par[3]),par[4],par[5],par[6],par[7],options.volt,options.cs,options.apix,options.sfile)
				ctffile.write( ctfparm )
				print "\t%s" % ( ctfparm )
			if options.drift:
				
				if options.show_fit: figureID=3
				else: figureID=0
				
				# now fit B-factor angular distribution to an ellipse
				driftpar = fitAstigDrift_by_Matlab(aniso_x, drift_y, B_mean, engineConnection = H)

				drift_mean = driftpar[0]	# use the mean defocus and convert to the same sign convention: negative for under defocus
				drift_amp = driftpar[1]		# use the mean defocus and convert to the same sign convention: negative for under defocus
				drift_ratio = drift_amp/fabs(drift_mean)
				drift_theta = driftpar[2]+math.pi/4	# covert to convention: counter-clockwise from +X is positive. EMAN ctfit use the larger B-factor direction as the major axis direction

				print "\tDrift: mean B factor = %g\tamp = %g\tratio = %g\tlarger B factor angle = %g" % (drift_mean, drift_amp, drift_ratio, drift_theta*180./math.pi)
		if options.pause<0: raw_input("Press return key to continue:")
		elif options.pause>0: time.sleep(options.pause)
	
	if H: pymat.close(H)
	ctffile.close()
	
	LOGend()

def parse_command_line():
	usage="%prog <*.img>|<*.hdf>|<*.tnf> [options]"
	parser = OptionParser(usage=usage, version="%prog v1.0, April 2004. By Wen Jiang <wjiang@bcm.tmc.edu> and Chao Yang <cyang@lbl.gov>")
	parser.add_option("--sf",dest="sfile",type="string",metavar="<structural factor file>",help="the structural factor curve file. use constant structural factors if this file is not given", default="")
	parser.add_option("--fitsf",dest="fitsfile",type="string",metavar="<structural factor file>",help="fit the structural factor curve from the images and save to the file", default="")
	parser.add_option("--apix",dest="apix",type="float",metavar="<Angstrom/pixel>",help="[Required] the sampling of the image, in unit of Angstrom/pixel")
	parser.add_option("--cs",dest="cs",type="float",metavar="<Cs>",help="[Required] the microscope's spherical abberation, in unit of mm")
	parser.add_option("--voltage",dest="volt",type="float",metavar="<voltage>",help="[Required] the microscope's accelerating voltage, in unit of kV")
	parser.add_option("--min_s",dest="lcut",type="float",metavar="<minimal spatial freq>",help="the minimal spatial frequency to include in fitting, in unit of 1/Angstrom. default to 0.01",default=0.01)
	parser.add_option("--max_s",dest="hcut",type="float",metavar="<maximal spatial freq>",help="the maximal spatial frequency to include in fitting, in unit of 1/Angstrom. default to 1.0",default=1.0)
	parser.add_option("--min_defocus",dest="minf",metavar="<minimal defocus>",type="float",help="the closest focus to consider, in unit of micrometer and positive for underdefocus. default to 0",default=0.0)
	parser.add_option("--max_defocus",dest="maxf",metavar="<maximal defocus>",type="float",help="the largest focus to consider, in unit of micrometer and positive for underdefocus. default to 4",default=4.0)
	parser.add_option("--max_bfactor",dest="maxB",metavar="<maximal B-factor>",type="float",help="the largest B-factor to consider, in unit of Angstrom^2. default to 1000",default=1000.0)
	parser.add_option("--Bexp",dest="Bexp",action="store_true",help="to use exponential B factor instead of the default Gaussian decay", default=0)
	parser.add_option("--min_ac",dest="min_ac",type="float",metavar="<minimal amplitude contrast>",help="the smallest amplitude contrast to consider, in range [0-1]. default to 0",default=0.0)
	parser.add_option("--max_ac",dest="max_ac",type="float",metavar="<maximal amplitude contrast>",help="the largest amplitude contrast to consider, in range [0-1]. default to 0.2",default=0.2)
	parser.add_option("--oversample",dest="over",type="int",metavar="<n>",help="oversample <n> times the power spectrum so there is enough sampling points to fit for small particles. only used when inputs are individual particle images. default to 1 to disable",default=1)
	parser.add_option("--gridbox",dest="gridbox",type="int",metavar="<box size>",help="if perform grid boxing first for a large micrograph/ccd image. default to 0 to disable",default=0)
	parser.add_option("--anisotropy_bins",dest="anisobins",type="int",metavar="<number of angular bins>",help="the number of angular bins (>8) to fit astigmatism and drift. default to 0 to disable astigmatism/drift fitting",default=0)
	parser.add_option("--ctfparm",dest="ctffile",type="string",metavar="<output parameter file>",help="the output filename for the fitting results to save. default to ctfparm.txt",default="ctfparm.txt")
	parser.add_option("--astigmatism",dest="astig",action="store_true",help="to fit astigmatism. must also set anisotropy_bins>=8 to activate. disabled by default")
	parser.add_option("--drift",dest="drift",action="store_true",help="to fit drift. must also set anisotropy_bins>=8 to activate. disabled by default")
	parser.add_option("--pause",dest="pause",type="int",metavar="<seconds>",help="to pause the fitting for the specified number of seconds after each micrograph, -1 will stop and wait for keyboard return to continue. default to 0 to disable pausing",default=0)
	parser.add_option("--show_fit",dest="show_fit",action="store_true",help="to show the fitting plot during the fitting process. disabled by default")
	parser.add_option("--keep_pwr",dest="keep",action="store_true",help="to keep the power spectrum files (*.pwr). disabled by default")
	parser.add_option("--useoldmfile",dest="useoldmfile",action="store_true",help="won't write new Matlab .m scripts and use exisiting scripts. disabled by default to output new scripts")
	parser.add_option("--useMatlabEngine",dest="useMatlabEngine",action="store_true",help="will use Matlab engine connection. disabled by default to directly run Matlab")
	
	if len(sys.argv)==1: 
		parser.print_help()
		sys.exit(-1)
	
	(options, args)=parser.parse_args()

	if not len(args):
		parser.print_help()
		print "\nPlease specify the images to fit"
		sys.exit(-1)
	
	if not options.apix or not options.cs or not options.volt:
		parser.print_help()
		print "\nPlease provide the required options"
		sys.exit(-1)

	if options.sfile and options.fitsfile:
		parser.print_help()
		print "\nPlease specify only \"--sf\" or \"--fitsf\" options"
		sys.exit(-1)
	
	if options.sfile and not os.path.exists(options.sfile):
		parser.print_help()
		print "\nPlease specifiy correct sf file, the specified sf file %s does not exist" % ( options.sfile )
		sys.exit(-1)
		
	if not options.over or options.over<1: options.over=1

	if (options.astig or options.drift) and options.anisobins<8:
		print "Warning: anisotropy bins is too small (%d) and is ignored. it should be >=8" % ( options.anisobins )
		options.anisobins=0
		options.astig=0
		options.drift=0

	if options.show_fit: options.show_fit=1
	else: options.show_fit=0
	
	if options.gridbox and options.gridbox < 32:
		print "Warning: grid box size is too small (%d) and is is ignored. it should be >=32" % (options.gridbox)
		options.gridbox=0

	return (options,args)

def calcAverageFFT(filename, oversample=1):
	num = EMAN.fileCount(filename)[0]	
	img = EMAN.EMData()
	#img.readImage(file,0,1)	# read header only for now
	img.readImage(filename)	# the above line does not work due to bug in EMAN module
	fftavg = EMAN.EMData()
	if img.isComplex():
		fftavg.setSize(img.xSize(), img.ySize())
	else:
		fftavg.setSize(img.xSize()*oversample+2, img.ySize()*oversample)
	
	fftavg.setComplex(1)
	fftavg.zero()
	
	if not img.isComplex():
		print "\tComputing average 2-D power spectrum of %s," % ( filename ),
		for i in range(num):
			img.readImage(filename, i)
			img.applyMask(-1,6)
			img.normalize()
			img.add(-img.edgeMean())
			if oversample>1: 
				img = img.clip(img.xSize()/2*(1-oversample), img.ySize()/2*(1-oversample),img.xSize()*oversample, img.ySize()*oversample) 
			fft = img.doFFT()
			fft.multConst(fft.ySize()*oversample)
			fftavg.addIncoherent(fft)
		fftavg.multConst(1./math.sqrt(num))
		tnffile = "%s.tnf" % (os.path.splitext(os.path.basename(filename))[0])
		fftavg.writeImage(tnffile,-1)
		print "and save to %s" % (tnffile)
		LOGOutfile(os.getpid(),tnffile)
	elif num==1:
		fftavg.readImage(filename)
	else:
		print "%s: does not support multiple Fourier Transform images stored in a single file %s" % (sys.argv[0], filename)
		sys.exit(2)
	return fftavg
	
def calcPowerSpectrum(fftavg, bins = 0):
	ret = []
	
	# now we have got the 2-D FFT
	ps = [ ]	
	fftavg.calcRadDist(fftavg.ySize(), 0, 0.5, ps)	#  2 times oversampled	
	ret.append(copy.deepcopy(ps))

	if bins>1:
		for i in range(bins):
			ps = []
			fftavg.calcRadDist(fftavg.ySize(), 0, 0.5, ps, i*math.pi*2./bins, math.pi*2./bins)	#  2 times oversampled
			ret.append(copy.deepcopy(ps))
	return ret


def findAllImages(files = None):	# find all image files in current directory
	tfiles = []
	for f in files: tfiles += glob.glob(f)
	files =  tfiles

	mgraphs = {}
	
	if len(files):	
		for f in files:
			if os.path.exists(f):
				#mid = os.path.basename(os.path.splitext(f)[0])
				mid = os.path.basename(f).split(".")[0]
				mgraphs[mid] = f
			else:
				print "Warning: cannot find image file %s" % (f)
	#else:
	#	for m in glob.glob("*.tnf"):
	#		mid = os.path.basename(os.path.splitext(m)[0])
	#		mgraphs[mid] = m

	#	for m in glob.glob("*.img") + glob.glob("*.hdf"):
	#		mid = os.path.basename(os.path.splitext(m)[0])
	#		if not mgraphs.has_key(mid):
	#			mgraphs[mid] = m
	
	ret = []
	mids = mgraphs.keys()
	mids.sort()
	for i in range(len(mids)):
		ret.append((mids[i], mgraphs[mids[i]]))
	
	return ret
	
def gridBoxAverageFFT(filename, boxsize):
	if boxsize <=1:
		print "\tERROR: boxsize (%d) is too small" % (boxsize)
		sys.exit(1)
	if not os.path.exists(filename):
		print "\tERROR: image %s does not exists" % (filename)
		sys.exit(1)
	
	img=EMAN.EMData()
	img.readImage(filename)
	nx = img.xSize()
	ny = img.ySize()
	if img.isComplex():
		print "\tWARNING: image %s (%dx%d) is a FFT image, skipping grid boxing" % (filename, nx, ny)
		return calcAverageFFT(filename)
	elif nx==ny and boxsize>nx:
		print "\tWARNING: image %s size (%dx%d) is smaller than box size (%d). skipping grid boxing" % (filename, nx, ny, boxsize)
		return calcAverageFFT(filename)
	elif nx!=ny and (boxsize>nx or boxsize>ny):
		print "\tERROR: image %s size (%dx%d) is too small for grid boxing using box size (%d)" % (filename, nx, ny, boxsize)
		sys.exit(1)

	cols = nx/boxsize
	rows = ny/boxsize
	x0 = (nx-cols*boxsize)/(cols+1)
	y0 = (ny-rows*boxsize)/(rows+1)
	dx = x0+boxsize
	dy = y0+boxsize
	
	fftavg = EMAN.EMData()
	fftavg.setSize(boxsize+2,boxsize)
	fftavg.setComplex(1)
	fftavg.zero()
	
	print "\tGrid boxing and computing average 2-D power spectrum of %s," % ( filename ),
	
	for j in range(rows):
		for i in range(cols):
			bimg = img.clip(x0+i*dx,y0+j*dy,boxsize,boxsize)
			bimg.applyMask(-1,6)
			bimg.normalize()
			fft = bimg.doFFT()
			fft.multConst(fft.ySize())
			fftavg.addIncoherent(fft)
	fftavg.multConst(1./math.sqrt(rows*cols))
	tnffile = "%s.tnf" % (os.path.splitext(os.path.basename(filename))[0])
	fftavg.writeImage(tnffile,-1)
	print "and save to %s" % (tnffile)
	LOGOutfile(os.getpid(),tnffile)
	return fftavg

def fitsf_by_Matlab(powerSpectrum_files, options, engineConnection = None):
	cmd = "chdir \'%s\';\n" % ( os.getcwd() )
	cmd+= "apix = %g;\n" % ( options.apix )
	cmd+= "cs = %g;\n" % ( options.cs )
	cmd+= "volt = %g;\n" % ( options.volt )
	cmd+= "lcut = %g;\n" % ( options.lcut )
	cmd+= "hcut = %g;\n" % ( options.hcut )
	cmd+= "show_plot = %d;\n" % ( options.show_fit )
	cmd+= "jac = 0;\n"
	fnames = "fname = ['%s'" % ( powerSpectrum_files[0] )
	for f in powerSpectrum_files: fnames +="; '%s'" % ( f )
	fnames+= "];\n"
	cmd+= fnames
	cmd+= "rehash; [par,sf,xs] = fitsfall(fname, jac, cs, volt, lcut, hcut, show_plot);\n"
	cmd+= "sftmp = [xs, sf]; save %s -ascii sftmp;\n" % ( options.fitsfile )
	if engineConnection:
		import pymat
		pymat.eval(engineConnection, cmd)
	else:
		cmd+="exit\n"
		#pfp, w, e = os.popen3("matlab -nosplash -nodesktop -nojvm") 
		pfp = os.popen("matlab -nosplash -nodesktop -nojvm > /dev/null", 'w') 
		pfp.write(cmd)
		pfp.close()

def fitctf_by_Matlab(powerSpectrum_file, options, lb = None, ub = None, par0=None, numTries = 1, engineConnection = None):
	if not lb:
		lb_s = 'lb = [  -%f   0     0    %f -Inf -Inf    0 -Inf]' % (options.maxf, options.min_ac)
	else:
		lb_s = 'lb = %s' % ( lb )
	if not ub:
		#ub_s = 'ub = [  -%f  %f   Inf    %f  Inf  Inf  Inf  Inf]' % (options.minf, options.maxB, options.max_ac)
		ub_s = 'ub = [  -%f Inf   Inf    %f  Inf  Inf  Inf  Inf]' % (options.minf, options.max_ac)
	else:
		ub_s = 'ub = %s' % ( ub )
	if not par0: 
		par0_s = 'par0 = [ -1.0 100.0   1.0  %f  .0  .0  1.0  1.0]' % ( (options.min_ac+options.max_ac)/2.0 )
	else:
		par0_s = 'par0 = %s' % ( par0 )
		
	cmd = "chdir \'%s\';\n" % ( os.getcwd() )
	cmd+= "apix = %g;\n" % ( options.apix )
	cmd+= "cs = %g;\n" % ( options.cs )
	cmd+= "volt = %g;\n" % ( options.volt )
	cmd+= "lcut = %g;\n" % ( options.lcut )
	cmd+= "hcut = %g;\n" % ( options.hcut )
	cmd+= "show_plot = %d;\n" % ( options.show_fit )
	cmd+= "sfile = \'%s\';\n" % ( options.sfile )
	cmd+= "jac = %d;\n" % ( 1 )
	cmd+= "bexp = %d;\n" % ( options.Bexp )
	cmd+= "num_try = %d;\n" % ( numTries )
	cmd+= "%s;\n" % ( lb_s )
	cmd+= "%s;\n" % ( ub_s )
	cmd+= "%s;\n" % ( par0_s )
	cmd+= "prob = \'%s\';\n" % ( powerSpectrum_file )
	cmd+= "par = fitlogctfc(prob,sfile,apix,cs,volt,lcut,hcut,par0,lb,ub,num_try,show_plot,jac,bexp);\n"
	if engineConnection:
		import pymat
		pymat.eval(engineConnection, cmd)
		par = pymat.get(engineConnection,'par')[0]
		return par
	else:
		# write to a file and return the result by file
		parfile = powerSpectrum_file+".par"
		cmd+="save %s -ascii par;\n" % ( parfile )
		cmd+="exit\n"
		#pfp = os.popen("matlab -nosplash -nodesktop -nojvm", 'w') 
		pfp = os.popen("matlab -nosplash -nodesktop -nojvm > /dev/null", 'w') 
		pfp.write(cmd)
		pfp.close()
		parline = open(parfile,'r').readlines()
		par = [ float(s) for s in parline[0].split() ]
		os.remove(parfile)
		return par

def fitAstigDrift_by_Matlab(ang, f_ang, par0, engineConnection = None):
	cmd = "par0=[%f, 0, 0];\n" % (par0)
	cmd+= "x=%s;\n" % ( ang )
	cmd+= "y=%s;\n" % ( f_ang )
	cmd+= "par = fit_sin(x,y,par0);\n"
	if engineConnection:
		par = pymat.get(H, 'par')[0]
		return par
	else:
		# write to a file and return the result by file
		parfile = "angularDistFit.par"
		cmd+="save %s -ascii par;\n" % ( parfile )
		cmd+="exit\n"
		#pfp, x, x = os.popen3("matlab -nosplash -nodesktop -nojvm") 
		pfp = os.popen("matlab -nosplash -nodesktop -nojvm > /dev/null", 'w') 
		pfp.write(cmd)
		pfp.close()
		parline = open(parfile,'r').readlines()
		par = [ float(s) for s in parline[0].split() ]
		os.remove(parfile)
		return par
					
matlab_1d_ctf_fit = r"""%function [par, xs, ys, y2, fs] = fitlogctfc(prob,sfile,apix,cs,volt,lcut,hcut,par0,lb,ub,num_try,plot,jac,bexp)
function par = fitlogctfc(prob,sfile,apix,cs,volt,lcut,hcut,par0,lb,ub,num_try,show_plot,jac,bexp)
%
% usage [par, xs, ys, y2, fs] = fitlogctfc(prob,sfile,apix,cs,volt,lcut,hcut,par0,lb,ub,num_try,plot,jac)
%
% Use MATLAB nonlinear least squares fitter to estimate CTF parameters.
%
% This version assumes a structure factor is available.
% It uses fmincon() to solve a constrained NLS problem.
%
% load power spectra
% 

[x,y]=setprob(prob,apix); 
n = length(x);
x(n) = [];
y(n) = [];
%
% load the structure factor
%
Xscat = load(sfile);  
[xs, fs, id]=fscatter(x, Xscat(:,1), Xscat(:,2));
ys   = y(id);

id2  = find(xs > lcut & xs <= hcut); 
xs = xs(id2);
ys = ys(id2);
ylog = log(ys);
fs = fs(id2);

if (jac == 0)
   options = optimset('Display','iter','MaxIter',100,'MaxFunEvals',1000, ...
                      'LargeScale','off');
else
   %options = optimset('Display','iter','MaxIter',1000,'MaxFunEvals',10000, ...
   options = optimset('Display','off','MaxIter',100,'MaxSQPIter',100,'MaxFunEvals',1000, ...
                      'LargeScale','off', 'GradObj', 'on',...
                      'GradConstr','on');
end;

minerr = 1e30;
bestpar = par0;
bestdz0 = 0;

if (num_try > 1)
%	for i = 0:num_try
%		dz=lb(1)+(ub(1)-lb(1))/(2*num_try)+i*(ub(1)-lb(1))/num_try
	dz = lb(1);
	while dz < ub(1)
		%dz=-i/3-0.1;
		par0(1)= dz;
		[par,resnrm] = fmincon(@rnlogctf, par0, [], [], [], [], lb, ub, @lognscon, ...
                	   options, xs, ys, fs, cs, volt, bexp);

		y2 = fctf(par, xs, fs, cs, volt, bexp);
		err= norm(sqrt(xs).*(ys-y2));
		if ( err < minerr )
			minerr = err;
			bestpar= par;
			bestdz0= dz;
		end
		dz = dz + 0.25 + 1.0 * abs(dz)/10. ;
	end
else
	[par,resnrm] = fmincon(@rnlogctf, par0, [], [], [], [], lb, ub, @lognscon, ...
                   options, xs, ys, fs, cs, volt, bexp);

	y2 = fctf(par, xs, fs, cs, volt, bexp);
	minerr= norm(sqrt(xs).*(ys-y2));
	bestpar= par;
	bestdz0= par0(1);
end

par=bestpar;

y2 = fctf(par, xs, fs, cs, volt, bexp);

fprintf('best dz0  = %11.3e\treserr = %11.3e\n\n', bestdz0, minerr);
fprintf('dz  = %11.3e\n', par(1));
fprintf('bf  = %11.3e\n', par(2));
fprintf('amp = %11.3e\n', par(3));
fprintf('ac  = %11.3e\n', sin(par(4)));
fprintf('n1  = %11.3e\n', par(5));
fprintf('n2  = %11.3e\n', par(6));
fprintf('n3  = %11.3e\n', par(7));
fprintf('n4  = %11.3e\n', par(8));

if (show_plot)
	figure(1);
	scrsz = get(0,'ScreenSize');
	set(gcf,'Position',[0 scrsz(4)/3 scrsz(3) scrsz(4)*2/3])
	clf;
	plot(xs, log(ys),'r.')
	hold;
	plot(xs, log(y2));    
	yn = fnoise(par, xs);
	plot(xs, log(yn),'-.');
	legend('data','fit');
	title(prob);
	xlabel('freqency (s)');
	ylabel('intensity');
	
	%tmp = [xs, ys]
	%save 4775-matlab.pwr tmp -ASCII
	
	%[ppath,probname,probext,probv]=fileparts(prob);
	%command = sprintf('print -dtiff %s.tif\n', probname);
	%eval(command); 
end


function [resnrm, g] = rnlogctf(par, s, y, sf, cs, volt, bexp)
%
% usage: [resnrm, g] = rnlogctf(par, s, y, sf, cs, volt, bexp)
%
% CTF function, par is the parameter list, 
%               s   is the frequency
%               y   is the measured power spectra
%               sf  is the structure factor
% returns both the residual norm square (RNS) of and the gradient of RNS.

dz     = par(1); % defocus
bf     = par(2); % B-factor
amp    = par(3); % Amplitude
theta  = par(4); % CA ca = sin(theta)
n1     = par(5); % noise
n2     = par(6);
n3     = par(7);
n4     = par(8);

lambda = 12.2639/sqrt(volt*1000+0.97845*volt*volt);
gamma = -2*pi*( cs*2.5e6*lambda^3*s.^4 + dz*5000*lambda*s.^2 );
C = amp*sin(gamma + theta);
if (bexp)
	E = exp(-bf*s);
else
	E = exp(-bf*s.^2);
end
%N = n1*exp(n2*s + n3*s.^2 + n4*sqrt(s));
N = n3*exp( -(pi/2*n4*s).^2 - n2*s - n1*sqrt(s));

yf = sf.*(C.^2).*(E.^2) + N;
r = log(y) - log(yf);

%resnrm = r'*r;
s2 = sqrt(s);
resnrm = (s2.*r)'*(s2.*r);

% compute the Jacobian
J(:,1) = sf.*(E.^2).*(2*C).*( amp*cos(gamma+theta) ).*(-2*pi*5000*lambda*s.^2)./yf;
if (bexp)
	J(:,2) = sf.*(C.^2).*(2*E.^2).*(-s.^2)./yf;
else
	J(:,2) = sf.*(C.^2).*(2*E.^2).*(-s.^2)./yf;
end
J(:,3) = sf.*(sin(gamma+theta).^2).*(E.^2).*(2*amp)./yf;
J(:,4) = sf.*(E.^2).*(2*C).*( amp*cos(gamma+theta) )./yf;

%J(:,5) = exp(n2*s + n3*s.^2 + n4*sqrt(s))./yf;
%J(:,6) = s.*N./yf;
%J(:,7) = s.^2.*N./yf;
%J(:,8) = sqrt(s).*N./yf;

J(:,5) = -sqrt(s).*N./yf;
J(:,6) = -s.*N./yf;
J(:,7) = exp( -(pi/2*n4*s).^2 - n2*s - n1*sqrt(s))./yf;
J(:,8) = -n4*pi.^2/2*s.^2.*N./yf;

% form the gradient
%g = -J'*r;
g = -2*J'*(r.*s);


function [c, ceq, cg, ceqg] = lognscon(par, s, y, sf, cs, volt, bexp)
%
% usage: [c, ceq, cg, ceqg] = lognscon(par, s, y, sf, cs, volt)
%
% Noise constraint for the CTF function: Noise must be strictly
% below the power spectra.
% par is the parameter list, 
%               s   is the frequency
%               y   is the power spectra
%               sf  is the structure factor
%

% returns both the constrains and the gradients of the constraints
dz     = par(1); % defocus
bf     = par(2); % B-factor
amp    = par(3); % Amplitude
theta  = par(4); % CA ca = sin(theta)
n1     = par(5); % noise
n2     = par(6);
n3     = par(7);
n4     = par(8);

n = length(s);
lambda = 12.2639/sqrt(volt*1000+0.97845*volt*volt);
gamma = -2*pi*( cs*2.5e6*lambda^3*s.^4 + dz*5000*lambda*s.^2 );

%C = amp*( sqrt(1-ca^2)*sin(gamma) + ca*cos(gamma) );
C = amp*sin(gamma + theta);

if(bexp)
	E = exp(-bf*s);
else
	E = exp(-bf*s.^2);
end

%N   = n1*exp(n2*s + n3*s.^2 + n4*sqrt(s));
N = n3*exp( -(pi/2*n4*s).^2 - n2*s - n1*sqrt(s));
Nslope = N.*(-pi*pi*n4.*s/2-n2.*ones(n,1)-n1./(2.*sqrt(s)));

ys = C.^2.*E.^2.*sf + N;
%c   = [-(ys -  1e-3); N - y; Nslope]; 
c   = [-(ys -  1e-2); N - y]; 
ceq = [];

%
% compute the gradient of the inequality constraints
%
cg(1:n,1) = -sf.*(E.^2).*(2*C).*( amp*cos(gamma+theta) ).*(-2*pi*5000*lambda*s.^2);
if(bexp)
	cg(1:n,2) = -sf.*(C.^2).*(2*E.^2).*(-s.^2);
else
	cg(1:n,2) = -sf.*(C.^2).*(2*E.^2).*(-s.^2);
end
cg(1:n,3) = -sf.*(sin(gamma+theta).^2).*(E.^2).*(2*amp);
cg(1:n,4) = -sf.*(E.^2).*(2*C).*( amp*cos(gamma+theta) );

%cg(1:n,5) = -exp(n2*s + n3*s.^2 + n4*sqrt(s));
%cg(1:n,6) = -s.*N;
%cg(1:n,7) = -s.^2.*N;
%cg(1:n,8) = -sqrt(s).*N;

cg(1:n,5) = sqrt(s).*N;
cg(1:n,6) = s.*N;
cg(1:n,7) = -exp( -(pi/2*n4*s).^2 - n2*s - n1*sqrt(s));
cg(1:n,8) = n4*pi.^2/2*s.^2.*N;

%
cg(n+1:2*n,1:4) = 0;
%cg(n+1:2*n,5) = exp(n2*s + n3*s.^2 + n4*sqrt(s));
%cg(n+1:2*n,6) = s.*N;
%cg(n+1:2*n,7) = s.^2.*N;
%cg(n+1:2*n,8) = sqrt(s).*N;

cg(n+1:2*n,5) = -sqrt(s).*N;
cg(n+1:2*n,6) = -s.*N;
cg(n+1:2*n,7) = exp( -(pi/2*n4*s).^2 - n2*s - n1*sqrt(s));
cg(n+1:2*n,8) = -n4*pi.^2/2*s.^2.*N;

%cg(2*n+1:3*n,1:4) = 0;
%cg(2*n+1:3*n,5) = -Nslope ./(2*sqrt(s));
%cg(2*n+1:3*n,6) = -Nslope;
%cg(2*n+1:3*n,7) = Nslope.*exp( -(pi/2*n4*s).^2 - n2*s - n1*sqrt(s));
%cg(2*n+1:3*n,8) = -Nslope.*(pi*pi*n4*s/2);

cg = cg';
%
ceqg = [];


function [x, y] = setprob(name, apix)
%
% [x, y] = setprob(name); 
%
% name must be a string; 
%

%fullname = [name '.' 'pwr'];
%eval(['load ' fullname]);
%

%V = eval(name);
V = load(name);
x = V(:,1);
y = V(:,2); 

%
n = length(x);
%apix = 3.63;  

%s = genfreq(2*n,apix);
%x = s;


function s = genfreq(imgsize,apix)
%
% s = genfreq(imgsizek,apix)
%
h = 1/(apix*imgsize);
s = [h:h:(h*imgsize/2.0)]';



function [x1, y1, id] = fscatter(x, xs, ys);
%
% usage: [x1, y1, id] = fscatter(x, xs, ys);
%
% interpolate the X-ray scattering structure factor
% at frequencies specified by x using data available
% at frequencies xs.
%

[xsmin, ismin] = min(xs);
[xsmax, ismax] = max(xs);
%
if ( x(1) > xsmin) 
   xsmin = x(1);
end 

if (  x(length(x)) < xsmax )
   xsmax = x(length(x));
end;

id = find(x >= xsmin & x <= xsmax);
x1 = x(id);
nd = length(id);
for i = 1:nd
   dist = abs(xs-x1(i));
   [dmin, imin] = min(dist);
   if ( xs(imin) <= x1(i) ) 
      if ( (xs(imin+1) - xs(imin) > 1e-6) ) 
         slope = (ys(imin) - ys(imin+1)) / (xs(imin) - xs(imin+1));
         y1(i) = ys(imin) + slope*(x1(i) - xs(imin));
      else
         y1(i) = ys(imin);
      end;
   else
      if ( (xs(imin) - xs(imin-1) > 1e-6) ) 
         slope = (ys(imin) - ys(imin-1)) / (xs(imin) - xs(imin-1));
         y1(i) = ys(imin) + slope*(x1(i) - xs(imin));
      else
         y1(i) = ys(imin);
      end;
   end
end;

y1 = y1';



function y = fnoise(par,s)
%
% usage: y = fnoise(par,s)
%
% log of the noise
%

n1 = par(5);
n2 = par(6);
n3 = par(7);
n4 = par(8);

%y = n1*exp(n2*s + n3*s.^2 + n4*sqrt(s));
y = n3*exp( -(pi/2*n4*s).^2 - n2*s - n1*sqrt(s));



function y = fctf(par, s, sf, cs, volt, bexp)
%
% usage: y = fctf(par, s, sf, cs, volt)
%
% CTF function, par is the parameter list, 
%               s   is the frequency
%               sf  is the structure factor
%               cs  is the sphererical abberation
%               volt is the EM voltage

dz     = par(1); % defocus
bf     = par(2); % B-factor
amp    = par(3); % Amplitude
theta  = par(4); % CA ca = sin(theta)
n1     = par(5); % noise
n2     = par(6);
n3     = par(7);
n4     = par(8);

lambda = 12.2639/sqrt(volt*1000+0.97845*volt*volt);
gamma = -2*pi*( cs*2.5e6*lambda^3*s.^4/4 + dz*5000*lambda*s.^2 );
C = amp*sin(gamma + theta);
if(bexp)
	E = exp(-bf*s);
else
	E = exp(-bf*s.^2);
end
%N = n1*exp(n2*s + n3*s.^2 + n4*sqrt(s));
N = n3*exp( -(pi/2*n4*s).^2 - n2*s - n1*sqrt(s));

y = sf.*(C.^2) .* (E.^2) + N;
"""

matlab_fit_sin = r"""function par = fit_sin(x, y, par0)
%
options = optimset('Display','off','MaxFunEvals',10000,...
                   'LevenbergMarquardt','on',...
                   'LargeScale','on');

lb = [par0(1)-abs(par0(1))    0   -pi/4 ];
ub = [par0(1)+abs(par0(1))  abs(par0(1))  pi/4 ];

%par = lsqcurvefit(@fsin3, par0, x, y, lb, ub, options);
par = lsqcurvefit(@fsin3, par0, x, y);
if par(2)<0
	par(2) = -par(2)
	par(3) = par(3)-pi/2
end

%--------------------------------------------------------------------------
function y = fsin3(par,x)

y = par(1) + par(2)*sin(2*(x-par(3)));
"""

matlab_sf_fit=r"""function [par,fs,xs] = fitsfall(fname, jac, cs, volt, lcut, hcut, show_plot)
%
% usage: [par,fs,xs] = fitsfall(fname, jac, cs, volt, lcut, hcut, show_plot)
%
%  This version treats the structural factor as parameters
%  to be estimated.
%
%  OUTPUT:
%  par(1:8,j) contains the parameters associated with the j-th data set
%  as described in Ludtke's JSB paper
%
%  par(1) --- defocus;
%  par(2) --- B-factor
%  par(3) --- amplitude;
%  par(4) --- amplitude contrast, sin(theta);
%  par(5) --- n1 
%  par(6) --- n2
%  par(7) --- n3
%  par(8) --- n4
%
%  fs     --- contains the structural factor
%  xs     --- frequencies on which the approximation is made
%
%  INPUT:
%
%  jac  - 0, use finite difference approximation to the Jacobian
%         1, use analytical Jacobian
%  

% load the names of the data files.
[nset,mlen] = size(fname);
xdat = [];
ydat = [];
%
for jset = 1:nset
   %
   % load one data set
   %
   [x, y] = setprob(fname(jset,1:mlen));
   %
   n = length(x);
   % delete the last point
   %
   x(n) = [];
   y(n) = [];
   n = length(x);
   %
   % accumulate data points
   %
   id    = find(x>=lcut & x<=hcut);
   xs    = x(id);
   ys    = y(id);
   xdat  = [xdat; xs];
   ydat  = [ydat; ys];
end;
%
%  Now call a nonlinear least squares solver to 
%  compute all ctf parameters as well as the 
%  structural factor 
%
nfs = length(xs);
%
%  the parameter consists of amplitude, B-factor and structral factor
%
beta0 = abs(rand(8*nset+nfs,1)); % random initial guess, can we do 
                                 % better than this?
for j = 1:nset
   beta0((j-1)*8+1) = -2.0;
   beta0((j-1)*8+2) = 100.0;
end;
%
%  there is some difference between running matlab 5 and matlab 6
%
fprintf('Begin NLS optimization...\n');
%lb = [-9 0    0   0.001 -inf -inf -inf -inf];
%ub = [0  inf inf  0.5    inf  inf  inf  inf];
lb = [];
ub = [];
if (jac == 0) 
   options = optimset('Display','iter','MaxFunEvals',200000,'MaxIter',2000,...
                      'LevenbergMarquardt','on','LargeScale','off');
   beta  = lsqcurvefit(@fsfctf,beta0, xdat, ydat, lb, ub, options, nset,...
                       cs, volt);   
else 
   options = optimset('Display','iter','MaxFunEvals',200000,...
                      'LevenbergMarquardt','on','LargeScale','off');
   beta  = lsqcurvefit(@fsfctfjac,beta0, xdat, ydat, lb, ub, options, nset,...
                       cs, volt);   
end;
fs = beta(8*nset+1:8*nset+nfs);
for j = 1:nset
  par(1,j) = beta(8*(j-1)+1);
  par(2,j) = beta(8*(j-1)+2);
  par(3,j) = beta(8*(j-1)+3);
  par(4,j) = beta(8*(j-1)+4);
  par(5,j) = beta(8*(j-1)+5);
  par(6,j) = beta(8*(j-1)+6);
  par(7,j) = beta(8*(j-1)+7);
  par(8,j) = beta(8*(j-1)+8);
end;
fprintf('Completed optimization. Ready to plot. Hit any key to continue\n');
if (show_plot)
	figure(1);
	clf;
	title('Fitting the power spectrum');
	xlabel('frequency');
	ylabel('magnitude.^2');
	y2 = fsfctf(beta, xdat, nset, cs, volt);
	for j = 1:nset
	   fprintf('plotting %s ...\n', fname(j,:));
	   clf;
	   ibeg = (j-1)*nfs+1;
	   iend = j*nfs;
	   plot(xdat(ibeg:iend), ydat(ibeg:iend), 'r.')
	   hold;
	   plot(xdat(ibeg:iend), y2(ibeg:iend));
	   legend('data','fit');
	   pause
	end
end
%
% ------------------------------------------
%
function y = fsfctf(par, s, nset, cs, volt)
%
% usage: y = fsfctf(par, s, nset, cs, volt)
%
% CTF function, par  is the parameter list, 
%               s    is the frequency
%               nset is the number of data set
%
% Note that the structral factor is considered
% as a parameter

npar   = length(par);
lambda = 12.2639/sqrt(volt*1000+0.97845*volt*volt);
sf     = par(8*nset+1:npar);
n      = length(s)/nset;
if (n*nset ~= length(s))
   fprintf('fsfctf: data sets are not of the same size, abort!\n');
   return; 
end; 
%
for j = 1:nset
   %
   % pick out ctf parameters
   %
   dz     = par(8*(j-1)+1); % defocus
   bf     = par(8*(j-1)+2); % square root of the B-factor
   amp    = par(8*(j-1)+3); % Amplitude
   theta  = par(8*(j-1)+4); % ca = sin(theta)
   n1     = par(8*(j-1)+5); % noise
   n2     = par(8*(j-1)+6);
   n3     = par(8*(j-1)+7);
   n4     = par(8*j);
   ca     = sin(theta);
   %
   % pick out freqency
   %
   ss  = s(n*(j-1)+1:n*j);
   gamma = -2*pi*( cs*2.5e6*lambda^3*ss.^4 + dz*5000*lambda*ss.^2 );

   C = amp*sin(gamma+theta);
   %E = exp(-bf^2*ss.^2);
   E = exp(-bf*ss.^2);
   N = n1*exp(n2*ss + n3*ss.^2 + n4*sqrt(ss));
   y(n*(j-1)+1:n*j,1) = sf.^2.*(C.^2) .* (E.^2) + N;
end;
%
%---------------------
%
function [y, J] = fsfctfjac(par, s, nset, cs, volt)
%
% usage: [y, J] = fsfctfjac(par, s, nset, cs, volt);
%
% evaluate both the function value and
% the Jacobian at s using the structure
% factor provided in sf.
%
% par --- parameter list, 
% s   --- frequency
% sf  --- structure factor
%

npar   = length(par);
lambda = 12.2639/sqrt(volt*1000+0.97845*volt*volt);
sf     = par(8*nset+1:npar);
n      = length(s)/nset;
if (n*nset ~= length(s))
   fprintf('fsfctf: data sets are not of the same size, abort!\n');
   return; 
end; 

for j = 1:nset
   dz     = par(8*j-7); % defocus
   bf     = par(8*j-6); % B-factor
   amp    = par(8*j-5); % Amplitude
   theta  = par(8*j-4); % CA ca = sin(theta)
   n1     = par(8*j-3); % noise
   n2     = par(8*j-2);
   n3     = par(8*j-1);
   n4     = par(8*j);
   
   ss  = s(n*(j-1)+1:n*j);

   gamma = -2*pi*( cs*2.5e6*lambda^3*ss.^4 + dz*5000*lambda*ss.^2 );

   C = amp*sin(gamma + theta);
   E = exp(-bf*ss.^2);
   N = n1*exp(n2*ss + n3*ss.^2 + n4*sqrt(ss));
   %N = n3*exp(-(pi*n4/2.0*ss).^2 -n2*ss - n1*sqrt(ss));

   y((j-1)*n+1:j*n,1) = sf.^2.*(C.^2) .* (E.^2) + N;

   J(1:n,8*(j-1)+1) = sf.^2.*(E.^2).*(2*C).*( amp*cos(gamma+theta) )...
                         .*(-2*pi*5000*lambda*ss.^2);
   J(1:n,8*(j-1)+2) = sf.^2.*(C.^2).*(2*E.^2).*(-ss.^2);
   J(1:n,8*(j-1)+3) = sf.^2.*(sin(gamma+theta).^2).*(E.^2).*(2*amp);
   J(1:n,8*(j-1)+4) = sf.^2.*(E.^2).*(2*C).*( amp*cos(gamma+theta) );
   J(1:n,8*(j-1)+5) = exp(n2*ss + n3*ss.^2 + n4*sqrt(ss));
   J(1:n,8*(j-1)+6) = ss.*N;
   J(1:n,8*(j-1)+7) = ss.^2.*N;
   J(1:n,8*j)       = sqrt(ss).*N;
end;
J(1:n,8*nset+1:8*nset+n) = diag(2*sf.*(C.^2).*(E.^2));

function [x, y] = setprob(name)
%
% [x, y] = setprob(name); 
%
% name must be a string (without suffix); 
%
fullname = [name '.' 'pwr']
%eval(['load' ' ' fullname])
V = load(fullname)
x = V(:,1);
y = V(:,2); 
"""

def LOGbegin(ARGV):
	if sys.platform == "win32": os.getppid = os.getpid
	out=open(".emanlog","a")
	b=string.split(ARGV[0],'/')
	ARGV[0]=b[len(b)-1]
	out.write("%d\t%d\t1\t%d\t%s\n" % (os.getpid(),time.time(),os.getppid(),string.join(ARGV," ")))
	out.close()
	return os.getpid()
	
def LOGInfile(pid,file):
	out=open(".emanlog","a")
	out.write("%d\t%d\t%d\t%s\t\n" % (pid,time.time(),10,file))
	out.close()
	
def LOGOutfile(pid,file):
	out=open(".emanlog","a")
	out.write("%d\t%d\t%d\t%s\t\n" % (pid,time.time(),11,file))
	out.close()
	
def LOGend():
	out=open(".emanlog","a")
	out.write("%d\t%d\t2\t-\t\n" % (os.getpid(),time.time()))
	out.close()
	
if __name__== "__main__":
	main()
